package Revision;
use strict;
use Rcs::Agent;
use MT::FileMgr;
use MT::Blog;
use MT::Request;
use MT::Template::Context;
use MT::ConfigMgr;
use Data::Dumper;
use vars qw($VERSION $mt_entry_save_handler $r);

MT::Template::Context->add_tag(RevisionLink => sub { &RevisionLink; });
MT::Template::Context->add_conditional_tag(IfRevised => sub { &IfRevised; });

$VERSION = '0.9';
$r = MT::Request->instance;
$r->cache('rcscount', 0);

# This intercepts the movable type Save routine.  It calls my own save,
# and then allows regular save to be called again.
#
# Future plugins won't overwrite this functionality if they use the same
# technique.  However, if they just manually redefine save by copying
# rather than calling the already-defined functionality, then they'll
# mess things up.
require MT::Entry;

{
	# Trap ugly redefinition warnings
	local $SIG{__WARN__} = sub { };

	$mt_entry_save_handler = \&MT::Entry::save;
	*MT::Entry::save = \&entrySave;
}

# conditional Movable Type plugin - returns true if there is a revision
# history of more than one revision.
sub IfRevised {
	my $ctx = shift;
    defined(my $entry = $ctx->stash('entry')) or return '';
	my $blog = MT::Blog->load($entry->blog_id);
	my $filepath = $blog->archive_path . "/" . $entry->id . ".rvs";
	my $rcs = new Rcs::Agent( file => $filepath );
	if (!$rcs->archived()) {
		return 0;
	}
	my $head = $rcs->head();
	if ($rcs->parent(revision => $head)) {
		return 1;
	}
	return 0;
}

# RevisionLink - returns the URL of the diff between the head version
# and the previous version.
sub RevisionLink {
	my $ctx = shift;
    defined(my $entry = $ctx->stash('entry')) or return '';
	my $blog = MT::Blog->load($entry->blog_id);
	my $filepath = $blog->archive_path . "/" . $entry->id . ".rvs";
	my $rcs = new Rcs::Agent( file => $filepath );
	if (!$rcs->archived()) {
		return '';
	}
	my $head = $rcs->head();
	my $cgipath = MT::ConfigMgr->instance->CGIPath;
	if (my $prev = $rcs->parent(revision => $head)) {
		return qq{<a href="} . $cgipath .  qq{mt-revision.cgi} .
			qq{?entry_id=} . $entry->id .
			qq{&from=$prev&to=$head">$head</a>};
	}
	return '';
}

# Redefined Save routine - it makes sure the rcs directory exists, and
# checks in the changes once per request.  Saved changes are only
# $entry->text and $entry->text_more .
sub entrySave {
	my $entry = shift;
	$r->cache('rcscount', $r->cache('rcscount') + 1);
	require MT::Entry;
	if ($r->cache('rcscount') == 1) {
		my $blog = MT::Blog->load($entry->blog_id);
		my $rcspath = $blog->archive_path . "/RCS";
		my $fmgr = $blog->file_mgr;
		if (!$fmgr->exists($rcspath)) {
			$fmgr->mkpath($rcspath);
		}
		my $text = $entry->text . "\n";
		if ($entry->text_more) {
			$text .= $entry->text_more . "\n";
		}
		my $filepath = $blog->archive_path . "/" . $entry->id . ".rvs";
		$fmgr->put_data($text, $filepath);
		my $rcs = new Rcs::Agent( file => $filepath );
		if (!$rcs->archived()) {
			$rcs->initialize();
		}
		$rcs->lock();
		$rcs->checkin(checkout=>"no");
	}
	return &$mt_entry_save_handler($entry);
}
