package keenworks::Revision;
use vars qw($VERSION);

$VERSION = '0.9';

use MT::App;
use MT::Blog;
use MT::Entry;
use MT::Template;
use MT::Template::Context;
#use Data::Dumper;
use Rcs::Agent;
use HTML::Diff;
use HTML::TokeParser;
@ISA = qw( MT::App );
use strict;

sub init {
	my $app = shift;
	$app->SUPER::init(@_) or return;
	$app->add_methods(main => \&diff);
	$app->{default_mode} = 'main';
	$app;
}

# {{{ Diff Documentation

=head1 Diff Documentation

Receive the 'entry_id', 'from', and 'to' cgi arguments from mt-revision.cgi.

Load the entry from the database and get the blog id.  Find the location
of the RCS revision files and create if necessary.

The browser always diffs between two ensuing revisions, so generate the
links if there are previous and next revisions to compare to.

Get the diff in unified format, and create a stack of "removed" and
"added" lines.

When the diff in question is over, construct an HTML string out of it
using HTML::Diff.  Repeat and append.

Load the individual archive entry page, but instead of displaying the
entry text, display the diff string instead.

=cut

# }}}
sub diff {
	my $app = shift;
	my $q = $app->{query};
	my ($entry_id, $from, $to) = 
		map $q->param($_), qw( entry_id from to );
    my $entry = MT::Entry->load($entry_id)
        or return $app->error("Invalid entry ID $entry_id");
	my $blog = MT::Blog->load($entry->blog_id);
	my $conv = defined $entry->convert_breaks ? $entry->convert_breaks :
			$blog->convert_paras;
	my $blog_id = $entry->blog_id;
	my $filepath = $blog->archive_path . "/" . $entry_id . ".rvs";
	my $rcs = new Rcs::Agent( file => $filepath );
	my $prevlink;
	if (my $prev = $rcs->parent(revision => $from)) {
		$prevlink = qq{<a href="} . $app->uri .
			qq{?entry_id=$entry_id} .
			qq{&from=$prev&to=$from">$prev -> $from</a>};
	}
	my $nextlink;
	if (my $next = $rcs->child(revision => $to)) {
		$nextlink = qq{<a href="/mt/mt-revision.cgi} .
			qq{?entry_id=$entry_id} .
			qq{&from=$to&to=$next">$to -> $next</a>};
	}
	return $app->error("No archive file exists!") if !$rcs->archived();
	my $diff = $rcs->diff(
			format => "unified", 
			revision1 => $from,
			revision2 => $to );
	my $r1 = shift @$diff;
	my $r2 = shift @$diff;
	my $str;	# eventual html
	my $old;	# arrayref for old revision
	my $new;	# arrayref for new revision
	my $tststr;	# actual diff txt
	foreach (@$diff) {
		#$tststr .= "$_\n";		# save diff txt (temporary)

		# save revision strings
		if (m/^\@\@(.*)\@\@$/) {
			$str .= qq{<p id="rev">$1</p>};
		}
		elsif (m/^\-(.*)$/) {
			push @$old, $conv ? 
				MT->apply_text_filters($1, $entry->text_filters) :
				$1;
		}
		elsif (m/^\+(.*)$/) {
			push @$new, $conv ?
				MT->apply_text_filters($1, $entry->text_filters) :
				$1;
		}
		else {
			my $blah = $_;
			my $tst = $conv ? 
				MT->apply_text_filters($blah, $entry->text_filters) :
				$blah;
			$str .= getHTMLDiff($old, $new) . $tst;
			undef $old;
			undef $new;
		}
	}
	$str .= getHTMLDiff($old, $new);
	my $p = HTML::TokeParser->new(\$str) || die "Can't process: $!";
	my @tags;
	my @msg;
	# crazy experimental attempt to add on unclosed tags in reverse
	# order
	while (my $token = $p->get_token) {
		if ($token->[0] eq "S") {
			#die "start, $token->[1]";
			#push @msg, "start tag: $token->[1]";
			if ($token->[1] !~ /^p|li|br$/) {
				push @tags, $token->[1];
				#push @msg, "pushed $token->[1]";
				#die "blah $token->[1]";
			}
		}
		elsif ($token->[0] eq "E") {
			my $j = 0;
			#push @msg, "end tag: $token->[1]";
			for (my $i = $#tags; $i>=0; $i--) {
				$j--;
				#push @msg, "searching tags, i=$i, j=$j, on $tags[$i]";
				if ($tags[$i] eq $token->[1]) {
					#push @msg, "match, splicing $token->[1] onward";
					splice(@tags, $j);
					last;
				}
			}
			#die "end, $token->[1]";
		}
	}
	while (@tags) {
		my $el = pop @tags;
		$str .= "</$el>";
	}
	#if (@tags) { die Dumper @tags; }
	#die Dumper @msg;
=blah
<div>
<br/>
Source:<br>
<textarea rows=12 cols=80>
$str
</textarea>
<br/><br/>
RCS:<br>
<textarea rows=12 cols=80>
$tststr
</textarea>
</div>
=cut

	my $str2 = qq{
<div class="diff">
<span class="del">removed</span><BR />
<span class="ins">added</span><BR />
$str
</div>
};
	$str2 .= qq{<div id="menu">};
	if ($prevlink) { $str2 .= qq{Previous: $prevlink<BR>}; }
	if ($nextlink) { $str2 .= qq{Next: $nextlink<BR>}; }
	$str2 .= qq{</div>};
	
	$app->{__blog_id} = $blog_id;
    return $app->error("Entry $entry_id is not published")
        unless $entry->status == MT::Entry::RELEASE();
    my $ctx = MT::Template::Context->new;
    $ctx->{current_archive_type} = 'Individual';
    $ctx->{current_timestamp} = $entry->created_on;
	$entry->text($str2);
	$entry->text_more("");
	$entry->convert_breaks(0);
	#die Dumper $entry;
    $ctx->stash('entry', $entry);
    my %cond = (
        EntryIfAllowComments => $entry->allow_comments,
        EntryIfCommentsOpen => $entry->allow_comments eq '1',
        EntryIfAllowPings => $entry->allow_pings,
        EntryIfExtended => $entry->text_more ? 1 : 0,
    );
    require MT::TemplateMap;
    my $map = MT::TemplateMap->load({ archive_type => 'Individual',
                                      blog_id => $app->{__blog_id},
                                      is_preferred => 1 });
    my $tmpl = MT::Template->load($map->template_id)
        or return $app->error("Can't load template " . $map->template_id);
    my $out = $tmpl->build($ctx, \%cond)
        or return $app->error("Building archive failed: " . $tmpl->errstr);
    return $out;
}

sub getHTMLDiff {
	my ($old, $new) = @_;
	my $str1 = "";
	my $str2 = "";
	#my $str1;
	#my $str2;
	my $str = "";
	my $res;
	if ($old) {
		$str1 = join "\n", @$old;
	}
	if ($new) {
		$str2 = join "\n", @$new;
	}
	if ($str1 or $str2) {
		$res = html_word_diff($str1, $str2);
		#die ("$str1, $str2");
		foreach my $el (@$res) {
			my ($code, $left, $right) = @$el;
			if ($code eq "u") {
				$str .= $left;
			}
			elsif ($code eq "+") {
				if ($left) {
					$str .= getHTML("span", "del", $left);
				}
				$str .= getHTML("span", "ins", $right);
			}
			elsif ($code eq "-") {
				if ($right) {
					$str .= getHTML("span", "ins", $right);
				}
				$str .= getHTML("span", "del", $left);
			}
			elsif ($code eq "c") {
				$str .= getHTML("span", "del", $left);
				$str .= getHTML("span", "ins", $right);
			}
		}
	}
	return $str;
}

sub getHTML {
	my ($tag, $class, $str) = @_;
	my $p = HTML::TokeParser->new(\$str) || die "Can't process: $!";
	my $chunk;
	while (my $token = $p->get_token) {
		if ($token->[0] eq "S") {
			$chunk .= $token->[4];
		}
		elsif ($token->[0] eq "E") {
			$chunk .= $token->[2];
		}
		elsif ($token->[0] eq "C") {
			$chunk .= $token->[1];
		}
		elsif ($token->[0] eq "D") {
			$chunk .= $token->[1];
		}
		elsif ($token->[0] eq "PI") {
			$chunk .= $token->[2];
		}
		elsif ($token->[0] eq "T") {
			my $txt = $token->[1];
			SUBTOKEN: while (my $subtoken = $p->get_token) {
				if ($subtoken->[0] eq "T") {
					$txt .= $subtoken->[1];
				}
				else {
					$p->unget_token($subtoken);
					last SUBTOKEN;
				}
			}
			$chunk .= qq{<$tag class="$class">} . $txt .  qq{</$tag>};
		}
	}
	return $chunk;
}

# vim: set foldmethod=marker:

1;
